<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use App\Http\Resources\MessagesResource;
use App\Http\Resources\RoomResource;
use App\Models\Message;
use App\Models\Room;
use App\Models\UserToken;
use App\User;
use App\Models\Notification;
use Edujugon\PushNotification\PushNotification;
use Illuminate\Http\Request;
use JWTAuth;
use Tymon\JWTAuth\Exceptions\JWTException;

class ChatController extends Controller
{

    public function check_room(Request $request)
    {
        $lang = $request->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
        }
        $validator = \Validator::make($request->all(), [
            'user_id' => 'required',
        ]);
        if ($validator->passes()) {

            $check = Room::where(['sender_id' => $request->user_id, 'receiver_id' => $user->id])->orWhere(['sender_id' => $user->id, 'receiver_id' => $request->user_id])->first();

            if ($check) {
                $message = Message::where('room_id', $check->id)->get();
                $data = MessagesResource::collection($message);
                return response()->json(['value' => true, 'data' => $data, 'room_id' => $check->id, 'code' => 200]);
            } else {
                $room = new Room();
                $room->sender_id = $user->id;
                $room->receiver_id = $request->user_id;
                $room->name = uniqid();
                $room->status = 'open';
                $room->save();

                return response()->json(['value' => true, 'data' => [], 'room_id' => $room->id, 'code' => 200]);

            }
        }
        return response()->json(['value' => true, 'msg' => $validator->errors()->first(), 'code' => 422]);
    }

    public function my_rooms(Request $request)
    {
        $lang = $request->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
        }
        $rooms = Room::where(['sender_id' => $user->id, 'status' => 'open'])->orWhere(['receiver_id' => $user->id, 'status' => 'open'])->get();
        $data = RoomResource::collection($rooms);
        return response()->json(['value' => true, 'data' => $data, 'code' => 200]);
    }

    public function store(Request $request)
    {
        $lang = $request->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
        }
        $validator = \Validator::make($request->all(), [
            'body' => 'required',
            'receiver_id' => 'required',
            'room_id' => 'required'
        ]);
        if ($validator->passes()) {
            $add = new Message();
            $add->sender_id = $user->id;
            $add->receiver_id = $request->receiver_id;
            $add->room_id = $request->room_id;
            $add->body = $request->body;
            $add->save();
            $data = Message::where('room_id', $request->room_id)->get();
            return response()->json(['value' => true, 'data' => MessagesResource::collection($data), 'code' => 200]);
        }
        return response()->json(['value' => true, 'msg' => $validator->errors()->first(), 'code' => 422]);
    }


}
