<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use App\Http\Resources\ProfileResource;
use App\Http\Resources\UserResource;
use App\Mail\ResetCode;
use App\Models\Follower;
use App\Models\UserToken;
use App\Models\VerifiedMobiles;
use App\User;
use Carbon\Carbon;
use Illuminate\Http\Request;
use JWTAuth;
use Tymon\JWTAuth\Exceptions\JWTException;

class UsersController extends Controller
{
    public function check_mobile(Request $request)
    {
        $lang = \request()->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
        }
        if ($user->verified == 'true') {
            return response()->json(['value' => true, 'data' => new UserResource($user), 'msg' => \Lang::get('data.documented'), 'code' => 200]);
        } else {
            $code = rand(1111, 9999);
            //ارسال رساله الموبايل فيما بعد
            $user->code = $code;
            $user->update();
            return response()->json(['value' => true, 'data' => new UserResource($user), 'msg' => \Lang::get('data.check_mobile'), 'code' => 200]);
        }
    }

    public function document_mobile(Request $request)
    {
        $lang = \request()->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
        }
        $validator = \Validator::make($request->all(), [
            'code' => 'required'
        ]);

        if ($validator->passes()) {
            if ($user->code == $request->code) {
                $user->verified = 'true';
                $user->code = null;
                $user->update();
                return response()->json(['value' => true, 'data' => new UserResource($user), 'msg' => \Lang::get('data.done_documented'), 'code' => 200]);
            } else {
                return response()->json(['value' => true, 'data' => new UserResource($user), 'msg' => \Lang::get('data.code_error'), 'code' => 200]);
            }
        }
        return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);

    }


    public function rate_user(Request $request)
    {
        $lang = \request()->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
        }
        $validator = \Validator::make($request->all(), [
            'rated_id' => 'required',
            'type' => 'required|in:good,bad',
            'details' => 'required',
            'buy_before' => 'required|in:true,false'
        ]);

        if ($validator->passes()) {
            if (UserRate::where(['user_id' => $user->id, 'rated_id' => $request->rated_id])->first()) {
                return response()->json(['value' => true, 'data' => \Lang::get('data.rate_again'), 'code' => 200]);
            }
            $user_date = Carbon::parse($user->created_at)->addMonths(3);
            if ($user_date > Carbon::now()) {
                return response()->json(['value' => true, 'data' => \Lang::get('data.rate_error'), 'code' => 200]);
            }
            $rate = new  UserRate();
            $rate->user_id = $user->id;
            $rate->rated_id = $request->rated_id;
            $rate->type = $request->type;
            $rate->details = $request->details;
            $rate->buy_before = $request->buy_before;
            $rate->save();
            return response()->json(['value' => true, 'data' => \Lang::get('data.rate'), 'code' => 200]);
        }
        return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
    }

    public function change_old_password(Request $request)
    {

        $lang = \request()->header('Accept-Language', 'ar');
        \App::setLocale($lang);

        $validator = \Validator::make($request->all(), [
            'old_password' => 'required',
            'new_password' => 'required|confirmed',
            'new_password_confirmation' => 'required'
        ]);
        if ($validator->passes()) {
            if (!$user = JWTAuth::parseToken()->authenticate()) {
                return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
            }

            if (\Hash::check($request['old_password'], $user->password)) {
                $user->password = bcrypt($request['new_password']);
                $user->update();
                $data = new UserResource($user);
                return response()->json(['value' => true, 'data' => $data, 'code' => 200]);
            } else {
                return response()->json(['value' => false, 'msg' => \Lang::get('data.password_not_match'), 'code' => 422]);
            }

        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
        }

    }

    public function black_list(Request $request)
    {
        $lang = $request->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        $validator = \Validator::make($request->all(), [
            'search' => ['required'],
        ]);
        if ($validator->passes()) {
            $rows = Block::where("mobile", "LIKE", "%" . $request['search'] . "%")->orWhere('id', $request['search'])->get();
            $data = BlackListUsersResource::collection($rows);
            return response()->json(['value' => true, 'data' => $data, 'code' => 200]);

        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
        }
    }

    public function register_mobile(Request $request)
    {
        $lang = $request->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        $validator = \Validator::make($request->all(), [
            'mobile' => ['required', 'unique:users'],
        ]);
        if ($validator->passes()) {

            $code = rand(1111, 9999);
            VerifiedMobiles::create([
                'mobile' => $request['mobile'],
                'code' => $code,
                'verified' => 'false'
            ]);
            // Code Mobile sms


            return response()->json(['value' => true, 'data' => \Lang::get('data.check_mobile'), 'code' => 200, 'mobile_code' => $code]);

        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
        }

    }

    public function check_code(Request $request)
    {
        $lang = $request->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        $validator = \Validator::make($request->all(),
            [
                'mobile' => 'required|exists:verified_mobiles,mobile',
                'code' => 'required'
            ]
        );
        if ($validator->passes()) {
            $check = VerifiedMobiles::where(['mobile' => $request['mobile'], 'code' => $request['code'], 'verified' => 'false'])->first();
            if ($check) {
                $check->verified = 'true';
                $check->update();
                return response()->json(['value' => true, 'data' => \Lang::get('data.code_correct'), 'code' => 500]);
            } else {
                return response()->json(['value' => false, 'msg' => \Lang::get('data.code_error'), 'code' => 422]);
            }
        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
        }
    }


    public function forget_password(Request $request)
    {
        $lang = $request->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        $validator = \Validator::make($request->all(), [
            'email' => 'email|required|exists:users,email'
        ]);
        if ($validator->passes()) {

            $user = User::whereEmail($request['email'])->first();
            $code = rand(1111, 9999);
            $user->code = $code;
            $user->update();
            // mobile code sms
            \Mail::to($user->email)->send(new ResetCode($code));
            $data = new UserResource($user);
            return response()->json(['value' => true, 'data' => $data, 'code' => 200]);

        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
        }
    }

    public function check_forget_code(Request $request)
    {
        $lang = $request->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        $validator = \Validator::make($request->all(), [
            'email' => 'email|required|exists:users,email',
            'code' => 'required|exists:users,code'
        ]);
        if ($validator->passes()) {
            $user = User::whereEmail($request['email'])->first();
            if (!$user) return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_mobile'), 'code' => 422]);
            if ($user->code != $request['code']) return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_code'), 'code' => 422]);
            $data = new UserResource($user);
            return response()->json(['value' => true, 'data' => $data, 'msg' => \Lang::get('data.correct_code'), 'code' => 200]);

        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
        }
    }

    public function change_password(Request $request)
    {
        $lang = $request->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        $validator = \Validator::make($request->all(), [
            'email' => 'email|required|exists:users,email',
            'password' => 'required|min:6',
            'device_token' => 'required',
            'device_type' => 'required|in:android,ios',
            'lat' => 'required',
            'lng' => 'required'
        ]);
        if ($validator->passes()) {

            $user = User::whereEmail($request['email'])->first();
            $user->password = bcrypt($request['password']);
            $user->update();
            $user->updateLocation($request['lat'], $request['lng']);
            $check = UserToken::where('token', $request['device_token'])->first();
            if ($check) {
                $check->status = 'connected';
                if ($request['device_type']) {
                    $check->device_type = $request['device_type'];
                }
                $check->update();
            } else {
                UserToken::create([
                    'user_id' => $user->id,
                    'token' => $request['device_token'],
                    'device_type' => $request['device_type'],
                    'status' => 'connected'
                ]);
            }
            $data = new UserResource($user);


            return response()->json(['value' => true, 'data' => $data, 'code' => 200]);
        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
        }
    }

    public function register(Request $request)
    {
        $lang = $request->header('Accept-Language', 'ar');
        \App::setLocale($lang);

        $validator = \Validator::make($request->all(), [
            'mobile' => ['nullable', 'unique:users'],
            'device_token' => 'required',
            'device_type' => 'required|in:ios,android',
            'city_id' => 'required|exists:cities,id',
            'password' => 'required|confirmed|min:6',
            'password_confirmation' => 'sometimes|required_with:password',
            'country_code_id' => 'nullable|exists:country_codes,id',
            'lat' => 'required',
            'lng' => 'required',
            'username' => 'required',
            'email' => 'required|email|unique:users'
        ]);
        if ($validator->passes()) {
            $user = new User();
            $user['mobile'] = $request['mobile'];
            $user['email'] = $request['email'];
            $user['name'] = $request['username'];
            $user['city_id'] = $request['city_id'];
            $user['lat'] = $request['lat'];
            $user['lng'] = $request['lng'];
            $user['password'] = bcrypt($request['password']);
            $user['online'] = 'true';
            $user['verified'] = 'true';
            $user['type'] = 'user';
            $user['status'] = 'active';
            $user['country_code_id'] = $request['country_code_id'];
            $user->save();
//
////            generate code and send it to user
//            $code = rand(1111, 9999);
//            $user->code = $code;
//            $user->update();
//            // send code sms


            $check = UserToken::where('token', $request['device_token'])->first();
            if ($check) {
                $check->status = 'connected';
                if ($request['device_type']) {
                    $check->device_type = $request['device_type'];
                }
                $check->update();
            } else {
                UserToken::create([
                    'user_id' => $user->id,
                    'token' => $request['device_token'],
                    'device_type' => $request['device_type'],
                    'status' => 'connected'
                ]);
            }

            $data = new UserResource($user);
            return response()->json(['value' => true, 'data' => $data, 'code' => 200]);
        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
        }
    }

    public function verified_register(Request $request)
    {
        $validator = \Validator::make($request->all(), [
            'mobile' => 'required|exists:users,mobile',
            'code' => 'required'
        ]);
        if ($validator->passes()) {
            $user = User::whereMobile($request['mobile'])->first();
            if ($user) {

                if ($user->code == $request['code']) {
                    $user->verified = 'true';
                    $user->update();
                    return response()->json(['value' => true, 'data' => \Lang::get('data.user_verified'), 'code' => 200]);
                } else {
                    return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_code'), 'code' => 422]);
                }
            } else {
                return response()->json(['value' => false, 'msg' => \Lang::get('data.no_user'), 'code' => 414]);
            }
        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
        }
    }

    public function login(Request $request)
    {
        $lang = $request->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        $validator = \Validator::make($request->all(), [
            'email' => ['required', 'exists:users,email'],
            'password' => 'required',
            'device_token' => 'required',
            'device_type' => 'required|in:ios,android',
            'lat' => 'required',
            'lng' => 'required'
        ]);
        if ($validator->passes()) {
            $auth = ['email' => $request->input('email'), 'password' => $request->input('password'), 'status' => 'active', 'type' => 'user'];
            try {
                if (!$token = JWTAuth::attempt($auth, ['exp' => Carbon::now()->addDays(7300)->timestamp])) {

                    return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_password')], 422);
                }
            } catch (JWTException $e) {
                return response()->json(['value' => false, 'msg' => 'could_not_create_token'], 500);
            }
            $user = User::where('email', $request['email'])->first();

            $user->updateLocation($request['lat'], $request['lng']);
            $check = UserToken::where('token', $request['device_token'])->first();
            if ($check) {
                $check->status = 'connected';
                if ($request['device_type']) {
                    $check->device_type = $request['device_type'];
                }
                $check->update();
            } else {
                UserToken::create([
                    'user_id' => $user->id,
                    'token' => $request['device_token'],
                    'device_type' => $request['device_type'],
                    'status' => 'connected'
                ]);
            }

            $data = new UserResource($user);
            return response()->json(['value' => true, 'data' => $data, 'code' => 200]);


        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
        }
    }

    public function logout(Request $request)
    {
        $lang = \request()->header('Accept-Language', 'ar');
        \App::setLocale($lang);

        $validator = \Validator::make($request->all(), [
            'device_token' => 'required|exists:user_tokens,token'
        ]);
        if ($validator->passes()) {
            if (!$user = JWTAuth::parseToken()->authenticate()) {
                return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
            }


            $exist_token = UserToken::where(['user_id' => $user->id, 'token' => $request['device_token']])->first();

            $exist_token->status = 'disconnected';
            $exist_token->update();

            $user->online = 'false';
            $user->update();
            JWTAuth::invalidate(JWTAuth::getToken());
            return response()->json(['value' => true, 'data' => \Lang::get('data.logout'), 'code' => 200]);

        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
        }


    }

    public function user_profile($id)
    {
        $user = User::find($id);
        if ($user) {
            $data = new ProfileResource($user);
            return response()->json(['value' => true, 'data' => $data, 'code' => 200]);
        } else {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_id'), 'code' => 422]);
        }
    }

    public function update_profile(Request $request)
    {
        $lang = \request()->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
        }
        $validator = \Validator::make($request->all(), [
            'image' => 'image',
            'lat' => 'required',
            'lng' => 'required',
            'name' => 'unique:users,name,' . $user->id,
            'email' => 'email|unique:users,email,' . $user->id,
            'mobile' => 'unique:users,mobile,' . $user->id,
            'city_id'=>'exists:cities,id'

        ]);
        if ($validator->passes()) {
            $user->name = $request['name'] ? $request['name'] : $user->name;
            $user->updateLocation($request['lat'], $request['lng']);
            $user->mobile = $request['mobile'] ? $request['mobile'] : $user->mobile;
            $user->email = $request['email'] ? $request['email'] : $user->email;
           $user->city_id = $request['city_id'] ? $request['city_id'] : $user->city_id;
            if ($request->hasFile('image')) {
                $user->setImageAttribute();
            }
            $user->update();
            $data = new UserResource($user);
            return response()->json(['value' => true, 'data' => $data, 'code' => 200]);
        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
        }


    }
    public function follow(Request $request){
        $lang = $request->header('Accept-Language','ar');
        \App::setLocale($lang);
        $validator = \Validator::make($request->all(),[
            'user_id'=>'required|exists:users,id'
        ]);
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
        }
        if ($validator->passes()){

            $check = Follower::where('user_id',$user->id)->where('follower_id',$request['user_id'])->first();
            if ($check){
                $check->delete();
                return response()->json(['value'=>true,'data'=>\Lang::get('data.unfollow'),'code'=>200]);
            }else{
                Follower::create([
                    'user_id'=>$user->id,
                    'follower_id'=>$request['user_id']
                ]);
                return response()->json(['value'=>true,'data'=>\Lang::get('data.follow'),'code'=>200]);

            }

        }else{
            return response()->json(['value'=>false,'msg'=>$validator->errors()->first(),'code'=>422]);
        }
    }


    public function getAuthenticatedUser()
    {
        try {

            if (!$user = JWTAuth::parseToken()->authenticate()) {
                return response()->json(['user_not_found'], 404);
            }

        } catch (Tymon\JWTAuth\Exceptions\TokenExpiredException $e) {

            return response()->json(['token_expired'], $e->getStatusCode());

        } catch (Tymon\JWTAuth\Exceptions\TokenInvalidException $e) {

            return response()->json(['token_invalid'], $e->getStatusCode());

        } catch (Tymon\JWTAuth\Exceptions\JWTException $e) {

            return response()->json(['token_absent'], $e->getStatusCode());

        }

        return response()->json(compact('user'));
    }

}
