<?php

namespace App\Http\Controllers\Web;

use App\Http\Controllers\Controller;
use App\Models\Favourite;
use App\Models\Product;
use App\Models\UserRate;
use App\User;
use Carbon\Carbon;
use Illuminate\Http\Request;

class UserController extends Controller
{
    public function profile()
    {
        $user = \Auth::user();
        $rows = \Auth::user()->product()->where('status', 'active')->paginate(9);
        return view('web.user.profile', compact('user', 'rows'));
    }

    public function change_password_ajax(Request $request)
    {
        $validator = \Validator::make($request->all(), [
            'old_password' => 'required',
            'password' => 'required|confirmed',
            'password_confirmation' => 'required'
        ],
            [
                'old_password.required' => \Lang::get('data.old_password_required'),
                'password.required' => \Lang::get('data.password_required'),
                'password.confirmed' => \Lang::get('data.password_confirmed'),
                'password_confirmation.required' => \Lang::get('data.password_confirmed_required'),
            ]);
        if ($validator->passes()) {
            $user = \Auth::user();
            if (\Hash::check($request['old_password'], $user->password)) {
                $user->password = bcrypt($request['password']);
                $user->update();

                return response()->json(['value' => true, 'data' => \Lang::get('data.password_changed'), 'code' => 200]);
            } else {
                return response()->json(['value' => false, 'msg' => \Lang::get('data.password_not_match'), 'code' => 422]);
            }
        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first()]);
        }
    }

    public function change_username_ajax(Request $request)
    {
        $validator = \Validator::make($request->all(), [

            'new_username' => 'required'
        ],
            [
                'new_username.required' => \Lang::get('data.new_username_required'),
            ]);
        if ($validator->passes()) {
            $user = \Auth::user();
            $user->name = $request['new_username'];
            $user->update();
            return response()->json(['value' => true, 'data' => \Lang::get('data.username_changed')]);
        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first()]);
        }
    }

    public function change_email_ajax(Request $request)
    {
        $user = \Auth::user();

        $validator = \Validator::make($request->all(), [

            'email' => 'required|email|unique:users,email,' . $user->id,
        ],
            [
                'email.required' => \Lang::get('data.email_required'),
                'email.unique' => \Lang::get('data.email_unique'),
            ]);
        if ($validator->passes()) {
            $user->email = $request['email'];
            $user->update();
            return response()->json(['value' => true, 'data' => \Lang::get('data.email_changed')]);
        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first()]);
        }
    }

    public function change_mobile_ajax(Request $request)
    {
        $user = \Auth::user();

        $validator = \Validator::make($request->all(), [

            'mobile' => 'required|unique:users,mobile,' . $user->id,
        ],
            [
                'mobile.required' => \Lang::get('data.mobile_required'),
                'mobile.unique' => \Lang::get('data.mobile_unique'),
            ]);
        if ($validator->passes()) {
            $user->email = $request['email'];
            $user->update();
            return response()->json(['value' => true, 'data' => \Lang::get('data.mobile_changed')]);
        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first()]);
        }
    }

    public function send_document_mobile(Request $request)
    {
        $mobile = \Auth::user()->mobile;
        $user = \Auth::user();
        // code to mobile
        $code = rand(1111, 9999);
        $user->verified_code = $code;
        $user->update();

        return response()->json(['value' => true, 'data' => \Lang::get('data.code_sent_to_mobile')]);

    }

    public function confirm_sent_code(Request $request)
    {
        $validator = \Validator::make($request->all(), [
            'code' => 'required'
        ]);
        if ($validator->passes()) {
            $user = \Auth::user();
            if ($user->verified_code == $request['code']) {
                $user->verified = 'true';
                $user->update();
                return response()->json(['value' => true, 'data' => \Lang::get('data.user_verified')]);

            } else {
                return response()->json(['value' => false, 'msg' => \Lang::get('data.code_error')]);
            }

        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first()]);
        }
    }

    public function upload_image_ajax(Request $request)
    {
//return $request->all();
        $validator = validator()->make($request->all(), ['image' => 'image']);
        if ($validator->passes()) {

            $user = \Auth::user();
//return $request->all();
//
            $user->setImageAttribute();
            $user->update();
            return response()->json(['value' => true, 'data' => \Lang::get('data.image_changed')]);

        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first()]);
        }

    }


    public function user_profile($mobile)
    {
        $check = User::whereMobile($mobile)->first();
        if(!$check) return redirect()->route('home')->with('error',\Lang::get('data.no_mobile_user'));

        if (\Auth::check()){
            $user = \Auth::user();
            if ($user->mobile == $mobile) {
                return redirect()->route('profile');
            } else {
                $user = User::whereMobile($mobile)->first();
                $rows = Product::whereUserId($user->id)->orderByDesc('id')->paginate(9);
                return view('web.user.user_profile', compact('user','rows'));
            }
        }else{
            $user = User::whereMobile($mobile)->first();
            $rows = Product::whereUserId($user->id)->orderByDesc('id')->paginate(9);
            return view('web.user.user_profile', compact('user','rows'));
        }


    }

    public function favourite()
    {
        $user = User::find(\Auth::user()->id);
        $rows = $user->favourite_products()->orderByDesc('id')->paginate('12');
        return view('web.user.favourite', compact('rows'));
    }

    public function add_remove_favourite(Request $request)
    {

        $row = Favourite::where(['user_id' => \Auth::user()->id, 'product_id' => $request['product_id']])->first();
        if ($row) {
            $row->delete();
            return response()->json(['status' => 1, 'msg' => \Lang::get('data.delete_favourite')]);
        } else {
            $favourite = new Favourite();
            $favourite->user_id = \Auth::user()->id;
            $favourite->product_id = $request['product_id'];
            $favourite->save();
            return response()->json(['status' => 0, 'msg' => \Lang::get('data.add_favourite')]);
        }
    }

    public function rate_user(Request $request)
    {
        $this->validate($request, [
            'user_mobile' => 'required|exists:users,mobile',
            'buy_before' => 'required|in:true,false',
            'type' => 'required|in:good,bad',
            'covenant' => 'required|accepted',
            'details' => 'required'
        ]);
        $user = \Auth::user();
        $user_date = Carbon::parse($user->created_at)->addMonths(3);
        if ($user_date > Carbon::now()) {
            return redirect()->back()->with('error', \Lang::get('data.rate_error'));
        }
        $rated_user = User::whereMobile($request['user_mobile'])->first();
        UserRate::create([
            'user_id' => $user->id,
            'rated_id' => $rated_user->id,
            'type' => $request['type'],
            'details' => $request['details'],
            'buy_before' => $request['buy_before'],
        ]);
        return redirect()->back()->with('message', \Lang::get('data.rate'));

    }

    public function follow_user_ajax(Request $request)
    {
        $validator = \Validator::make($request->all(), [
            'mobile' => 'required'
        ]);
        if ($validator->passes()) {

            $rate_user = User::whereMobile($request['mobile'])->first();


        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first()]);
        }
    }
}
